//***************************************************************************************** 
// Description: GPL951 I2C Test Code
// Plateform: GPL951, unSP IDE 
// Edited and Created by Generalplus Technology Co., Ltd.

// EEPROM : 24LC32
//    GPL951						Device
// IOC[2]    : I2C SCK				24LC32   : SCL (Pin6)
// IOC[3]    : I2C SDA				24LC32   : SDA (Pin5)
// GND								24LC32	 : A0	(Pin1)
// GND								24LC32	 : A1	(Pin2)
// GND								24LC32	 : A2	(Pin3)

// Update History:
// 2018/02/05 	V1.0 : Release
// 2019/04/19	V1.1 : Add page write function
// 2019/06/19	V1.2 : Modify I2C.asm : I2C Error function
//					   Add I2C Read / Write Error Flag (I2C_Error_Flag)
//*****************************************************************************************
// Header File Included Area
//*****************************************************************************************
#include	"GPL951_Body.h"
#include	"I2C.h"

//*****************************************************************************************
// Contant Defintion Area
//*****************************************************************************************


//*****************************************************************************************
// Global Variable Defintion Area
//*****************************************************************************************
int Data[1000];
int PageWriteSourceData[32];
int I2C_Error_Flag;

//*****************************************************************************************
// Function Call Definition Area
//*****************************************************************************************  
void main(void)
{
	unsigned i, j=0x0;
	I2C_IF_Initial();
	
// ====================== Byte Write, Random Read Test ======================
	for (i=0; i<=0xFF; i++) {
		EEPROM_ByteWrite(i, j);				//EEPROM_ByteWrite(Address, Data)
		ProgrammingWait();
		j++;
	}
	
	for (i=0; i<=0xFF; i++) {
		Data[i] = EEPROM_RandomRead(i);		//EEPROM_RandomRead(Address)
		while (Data[i] != ((i+0x0)&0xFF))
			*P_WatchDog_Clear = C_WDT_Clear;
	}
			
//===========================================================================	

// ========================= Sequential Read Test ===========================
	for (i=0; i<1000; i++)
		Data[i] = 0;
	i=0;
	EEPROM_SequentialRead(i,0x100);			//EEPROM_SequentialRead(Start Address, Data Length)
	
	for (i=0; i<=0xFF; i++) {
		while (Data[i] != ((i+0x0)&0xFF))
			*P_WatchDog_Clear = C_WDT_Clear;
	}	
//===========================================================================

// ======================= Page Write, Sequential Read ======================
	// 24LC32, 1 page 32 bytes, user need to reference spec to modify it 
	j = 0xFF;
	for (i=0; i<32; i++) {
		PageWriteSourceData[i] = j;
		j--;
	}	
	EEPROM_PageWrite(0x00);					// 0x0 ~ 0x1F : 32 Byte
	ProgrammingWait();						// Write cycle time > 5ms
	
	j = 0x80;
	for (i=0; i<32; i++) {
		PageWriteSourceData[i] = j;
		j--;
	}			
	EEPROM_PageWrite(0x20);					// 0x20 ~ 0x3F : 32 Byte
	ProgrammingWait();	
	
	for (i=0; i<1000; i++)
		Data[i] = 0;
	i=0;
	EEPROM_SequentialRead(i,64);			//EEPROM_SequentialRead(Start Address, Data Length)
//===========================================================================	
	
	while(1)
		*P_WatchDog_Clear = C_WDT_Clear;
}
